@@ -1,5 +1,5 @@
 // Licensed under the Apache License: http://www.apache.org/licenses/LICENSE-2.0
-// For details: https://bitbucket.org/ned/coveragepy/src/default/NOTICE.txt
+// For details: https://github.com/nedbat/coveragepy/blob/master/NOTICE.txt
 
 // Coverage.py HTML report browser code.
 /*jslint browser: true, sloppy: true, vars: true, plusplus: true, maxerr: 50, indent: 4 */
@@ -73,30 +73,28 @@ coverage.wire_up_filter = function () {
         }
         else {
             // Filter table items by value.
-            var hide = $([]);
-            var show = $([]);
+            var hidden = 0;
+            var shown = 0;
 
-            // Compile elements to hide / show.
+            // Hide / show elements.
             $.each(table_row_names, function () {
                 var element = $(this).parents("tr");
 
                 if ($(this).text().indexOf(filter_value) === -1) {
                     // hide
-                    hide = hide.add(element);
+                    element.addClass("hidden");
+                    hidden++;
                 }
                 else {
                     // show
-                    show = show.add(element);
+                    element.removeClass("hidden");
+                    shown++;
                 }
             });
 
-            // Perform DOM manipulation.
-            hide.addClass("hidden");
-            show.removeClass("hidden");
-
             // Show placeholder if no rows will be displayed.
             if (no_rows.length > 0) {
-                if (show.length === 0) {
+                if (shown === 0) {
                     // Show placeholder, hide table.
                     no_rows.show();
                     table.hide();
@@ -109,7 +107,7 @@ coverage.wire_up_filter = function () {
             }
 
             // Manage dynamic header:
-            if (hide.length > 0) {
+            if (hidden > 0) {
                 // Calculate new dynamic sum values based on visible rows.
                 for (var column = 2; column < 20; column++) {
                     // Calculate summed value.
@@ -171,22 +169,16 @@ coverage.wire_up_filter = function () {
 
 // Loaded on index.html
 coverage.index_ready = function ($) {
-    // Look for a cookie containing previous sort settings:
+    // Look for a localStorage item containing previous sort settings:
     var sort_list = [];
-    var cookie_name = "COVERAGE_INDEX_SORT";
-    var i;
+    var storage_name = "COVERAGE_INDEX_SORT";
+    var stored_list = undefined;
+    try {
+        stored_list = localStorage.getItem(storage_name);
+    } catch(err) {}
 
-    // This almost makes it worth installing the jQuery cookie plugin:
-    if (document.cookie.indexOf(cookie_name) > -1) {
-        var cookies = document.cookie.split(";");
-        for (i = 0; i < cookies.length; i++) {
-            var parts = cookies[i].split("=");
-
-            if ($.trim(parts[0]) === cookie_name && parts[1]) {
-                sort_list = eval("[[" + parts[1] + "]]");
-                break;
-            }
-        }
+    if (stored_list) {
+        sort_list = JSON.parse('[[' + stored_list + ']]');
     }
 
     // Create a new widget which exists only to save and restore
@@ -233,7 +225,9 @@ coverage.index_ready = function ($) {
 
     // Watch for page unload events so we can save the final sort settings:
     $(window).unload(function () {
-        document.cookie = cookie_name + "=" + sort_list.toString() + "; path=/";
+        try {
+            localStorage.setItem(storage_name, sort_list.toString())
+        } catch(err) {}
     });
 };
 
@@ -242,7 +236,7 @@ coverage.index_ready = function ($) {
 coverage.pyfile_ready = function ($) {
     // If we're directed to a particular line number, highlight the line.
     var frag = location.hash;
-    if (frag.length > 2 && frag[1] === 'n') {
+    if (frag.length > 2 && frag[1] === 't') {
         $(frag).addClass('highlight');
         coverage.set_sel(parseInt(frag.substr(2), 10));
     }
@@ -267,21 +261,22 @@ coverage.pyfile_ready = function ($) {
 
     coverage.init_scroll_markers();
 
-    // Rebuild scroll markers after window high changing
-    $(window).resize(coverage.resize_scroll_markers);
+    // Rebuild scroll markers when the window height changes.
+    $(window).resize(coverage.build_scroll_markers);
 };
 
 coverage.toggle_lines = function (btn, cls) {
     btn = $(btn);
-    var hide = "hide_"+cls;
-    if (btn.hasClass(hide)) {
-        $("#source ."+cls).removeClass(hide);
-        btn.removeClass(hide);
+    var show = "show_"+cls;
+    if (btn.hasClass(show)) {
+        $("#source ." + cls).removeClass(show);
+        btn.removeClass(show);
     }
     else {
-        $("#source ."+cls).addClass(hide);
-        btn.addClass(hide);
+        $("#source ." + cls).addClass(show);
+        btn.addClass(show);
     }
+    coverage.build_scroll_markers();
 };
 
 // Return the nth line div.
@@ -294,11 +289,6 @@ coverage.num_elt = function (n) {
     return $("#n" + n);
 };
 
-// Return the container of all the code.
-coverage.code_container = function () {
-    return $(".linenos");
-};
-
 // Set the selection.  b and e are line numbers.
 coverage.set_sel = function (b, e) {
     // The first line selected.
@@ -317,9 +307,17 @@ coverage.to_first_chunk = function () {
     coverage.to_next_chunk();
 };
 
-coverage.is_transparent = function (color) {
-    // Different browsers return different colors for "none".
-    return color === "transparent" || color === "rgba(0, 0, 0, 0)";
+// Return a string indicating what kind of chunk this line belongs to,
+// or null if not a chunk.
+coverage.chunk_indicator = function (line_elt) {
+    var klass = line_elt.attr('class');
+    if (klass) {
+        var m = klass.match(/\bshow_\w+\b/);
+        if (m) {
+            return m[0];
+        }
+    }
+    return null;
 };
 
 coverage.to_next_chunk = function () {
@@ -327,14 +325,14 @@ coverage.to_next_chunk = function () {
 
     // Find the start of the next colored chunk.
     var probe = c.sel_end;
-    var color, probe_line;
+    var chunk_indicator, probe_line;
     while (true) {
         probe_line = c.line_elt(probe);
         if (probe_line.length === 0) {
             return;
         }
-        color = probe_line.css("background-color");
-        if (!c.is_transparent(color)) {
+        chunk_indicator = c.chunk_indicator(probe_line);
+        if (chunk_indicator) {
             break;
         }
         probe++;
@@ -344,11 +342,11 @@ coverage.to_next_chunk = function () {
     var begin = probe;
 
     // Find the end of this chunk.
-    var next_color = color;
-    while (next_color === color) {
+    var next_indicator = chunk_indicator;
+    while (next_indicator === chunk_indicator) {
         probe++;
         probe_line = c.line_elt(probe);
-        next_color = probe_line.css("background-color");
+        next_indicator = c.chunk_indicator(probe_line);
     }
     c.set_sel(begin, probe);
     c.show_selection();
@@ -363,25 +361,25 @@ coverage.to_prev_chunk = function () {
     if (probe_line.length === 0) {
         return;
     }
-    var color = probe_line.css("background-color");
-    while (probe > 0 && c.is_transparent(color)) {
+    var chunk_indicator = c.chunk_indicator(probe_line);
+    while (probe > 0 && !chunk_indicator) {
         probe--;
         probe_line = c.line_elt(probe);
         if (probe_line.length === 0) {
             return;
         }
-        color = probe_line.css("background-color");
+        chunk_indicator = c.chunk_indicator(probe_line);
     }
 
     // There's a prev chunk, `probe` points to its last line.
     var end = probe+1;
 
     // Find the beginning of this chunk.
-    var prev_color = color;
-    while (prev_color === color) {
+    var prev_indicator = chunk_indicator;
+    while (prev_indicator === chunk_indicator) {
         probe--;
         probe_line = c.line_elt(probe);
-        prev_color = probe_line.css("background-color");
+        prev_indicator = c.chunk_indicator(probe_line);
     }
     c.set_sel(probe+1, end);
     c.show_selection();
@@ -453,29 +451,29 @@ coverage.select_line_or_chunk = function (lineno) {
     if (probe_line.length === 0) {
         return;
     }
-    var the_color = probe_line.css("background-color");
-    if (!c.is_transparent(the_color)) {
+    var the_indicator = c.chunk_indicator(probe_line);
+    if (the_indicator) {
         // The line is in a highlighted chunk.
         // Search backward for the first line.
         var probe = lineno;
-        var color = the_color;
-        while (probe > 0 && color === the_color) {
+        var indicator = the_indicator;
+        while (probe > 0 && indicator === the_indicator) {
             probe--;
             probe_line = c.line_elt(probe);
             if (probe_line.length === 0) {
                 break;
             }
-            color = probe_line.css("background-color");
+            indicator = c.chunk_indicator(probe_line);
         }
         var begin = probe + 1;
 
         // Search forward for the last line.
         probe = lineno;
-        color = the_color;
-        while (color === the_color) {
+        indicator = the_indicator;
+        while (indicator === the_indicator) {
             probe++;
             probe_line = c.line_elt(probe);
-            color = probe_line.css("background-color");
+            indicator = c.chunk_indicator(probe_line);
         }
 
         coverage.set_sel(begin, probe);
@@ -489,7 +487,7 @@ coverage.show_selection = function () {
     var c = coverage;
 
     // Highlight the lines in the chunk
-    c.code_container().find(".highlight").removeClass("highlight");
+    $(".linenos .highlight").removeClass("highlight");
     for (var probe = c.sel_begin; probe > 0 && probe < c.sel_end; probe++) {
         c.num_elt(probe).addClass("highlight");
     }
@@ -519,21 +517,21 @@ coverage.finish_scrolling = function () {
 coverage.init_scroll_markers = function () {
     var c = coverage;
     // Init some variables
-    c.lines_len = $('td.text p').length;
+    c.lines_len = $('#source p').length;
     c.body_h = $('body').height();
     c.header_h = $('div#header').height();
-    c.missed_lines = $('td.text p.mis, td.text p.par');
 
     // Build html
-    c.resize_scroll_markers();
+    c.build_scroll_markers();
 };
 
-coverage.resize_scroll_markers = function () {
+coverage.build_scroll_markers = function () {
     var c = coverage,
         min_line_height = 3,
         max_line_height = 10,
         visible_window_h = $(window).height();
 
+    c.lines_to_mark = $('#source').find('p.show_run, p.show_mis, p.show_exc, p.show_exc, p.show_par');
     $('#scroll_marker').remove();
     // Don't build markers if the window has no scroll bar.
     if (c.body_h <= visible_window_h) {
@@ -557,11 +555,16 @@ coverage.resize_scroll_markers = function () {
 
     var previous_line = -99,
         last_mark,
-        last_top;
+        last_top,
+        offsets = {};
 
-    c.missed_lines.each(function () {
-        var line_top = Math.round($(this).offset().top * marker_scale),
-            id_name = $(this).attr('id'),
+    // Calculate line offsets outside loop to prevent relayouts
+    c.lines_to_mark.each(function() {
+        offsets[this.id] = $(this).offset().top;
+    });
+    c.lines_to_mark.each(function () {
+        var id_name = $(this).attr('id'),
+            line_top = Math.round(offsets[id_name] * marker_scale),
             line_number = parseInt(id_name.substring(1, id_name.length));
 
         if (line_number === previous_line + 1) {
